﻿using System;

namespace Core
{
  public class ControlPort : IControlPort
  {
    #region IModel Members

    public event EventHandler CPUModeChanged;
    public event EventHandler MPModeChanged;
    public event EventHandler INPStateChanged;
    public event EventHandler DataChanged;
    public event EventHandler QStateChanged;

    private CPUMode cpuMode;
    public CPUMode CPUMode
    {
      get { return cpuMode; }
      set
      {
        if (cpuMode != value)
        {
          cpuMode = value;
          OnCPUModeChanged();
        }
      }
    }

    private MemoryProtect memoryProtect;
    public MemoryProtect MemoryProtect
    {
      get { return memoryProtect; }
      set
      {
        if (memoryProtect != value)
        {
          memoryProtect = value;
          OnMPModeChanged();
        }
      }
    }

    private InputState inputState;
    public InputState InputState
    {
      get { return inputState; }
      set
      {
        if (inputState != value)
        {
          inputState = value;
          OnInputStateChanged();
        }
      }
    }

    private string data;
    public string Data
    {
      get { return data; }
      set
      {
          data = value;
          OnDataChanged();
      }
    }

    private bool qState;
    public bool QState
    {
      get { return qState; }
      set
      {
        if (qState != value)
        {
          qState = value;
          OnQStateChanged();
        }
      }
    }

    public virtual void Start()
    {
      throw new NotImplementedException();
    }

    public virtual void Stop()
    {
      throw new NotImplementedException();
    }

    public virtual void SetCPUMode(CPUMode mode)
    {
      throw new NotImplementedException();
    }

    public virtual void SetMPState(MemoryProtect mp)
    {
      throw new NotImplementedException();
    }

    public virtual void SetINPState(InputState state)
    {
      throw new NotImplementedException();
    }

    public virtual void WriteData(string data)
    {
      throw new NotImplementedException();
    }

    #endregion
    
    protected virtual void OnCPUModeChanged()
    {
      if (CPUModeChanged != null)
      {
        CPUModeChanged(this, EventArgs.Empty);
      }
    }

    protected virtual void OnMPModeChanged()
    {
      if (MPModeChanged != null)
      {
        MPModeChanged(this, EventArgs.Empty);
      }
    }

    protected virtual void OnInputStateChanged()
    {
      if (INPStateChanged != null)
      {
        INPStateChanged(this, EventArgs.Empty);
      }
    }

    protected virtual void OnQStateChanged()
    {
      if (QStateChanged != null)
      {
        QStateChanged(this, EventArgs.Empty);
      }
    }

    protected virtual void OnDataChanged()
    {
      if (DataChanged != null)
      {
        DataChanged(this, EventArgs.Empty);
      }
    }
  }
}
