﻿using System;
using System.IO;
using System.Threading;

namespace Core
{
  public class Presenter
  {
    private IView view;
    private IControlPort model;

    string fileName = string.Empty;

    public Presenter()
    {
    }

    public Presenter(IView view)
    {
      this.view = view;
    }

    public Presenter(IView view, IControlPort model)
    {
      this.view = view;
      this.model = model;

      Initialize();
      CPUMode = CPUMode.Reset;
    }

    public CPUMode CPUMode
    {
      get { return model.CPUMode; }
      set { model.SetCPUMode(value); }
    }

    public MemoryProtect MemoryProtect
    {
      get { return model.MemoryProtect; }
      set { model.SetMPState(value); }
    }
    
    public InputState InputState
    {
      get { return model.InputState; }
      set { model.SetINPState(value); }
    }

    public string Data
    {
      get { return model.Data; }
    }

    public bool QState
    {
      get { return model.QState; }
    }

    public IControlPort ControlPort
    {
      get { return model; }
    }

    protected virtual void Initialize()
    {
      if (view == null)
      {
        return;
      }

      if (model == null)
      {
        return;
      }

      model.CPUModeChanged += new EventHandler(view.SetCPUMode);
      model.DataChanged += new EventHandler(view.SetDisplay);
      model.INPStateChanged += new EventHandler(view.SetInputState);
      model.MPModeChanged += new EventHandler(view.SetMPMode);
      model.QStateChanged += new EventHandler(view.SetQ);

      view.Presenter = this;
    }

    public void SendData(string data)
    {
      this.model.WriteData(data);
    }

    public void SetPort(IControlPort port)
    {
      this.model = port;
      Initialize();
    }

    public void SetView(IView view)
    {
      this.view = view;
      Initialize();
    }

    public void LoadFromFile(string fileName)
    {
      this.fileName = fileName;

      Thread loadThread = new Thread(new ThreadStart(DoRead));
      loadThread.Name = "Memory loading thread.";
      loadThread.IsBackground = true;

      loadThread.Start();
    }

    protected virtual void DoRead()
    {
      MemoryProtect = MemoryProtect.Disabled;
      System.Threading.Thread.Sleep(125);
      CPUMode = CPUMode.Load; 
        
      StreamReader reader = File.OpenText(fileName);
      string codeLine = string.Empty;
      string byteString = string.Empty;
      bool first = true;
      while (!reader.EndOfStream)
      {
        codeLine = reader.ReadLine();
        if (codeLine != string.Empty)
        {
          // Is this an Intel HEX data line?
          if ((codeLine[0] == ':') && (codeLine[8] == '0'))
          {
            for (int i = 9; i < codeLine.Length - 2; i++)
            {
              if (first)
              {
                byteString = codeLine[i].ToString();
                first = false;
              }
              else
              {
                byteString = byteString + codeLine[i].ToString();
                this.model.WriteData(byteString);
                System.Threading.Thread.Sleep(50);
                if (model.GetType() == typeof(OfflineControlPort))
                {
                  InputState = InputState.Down;
                  System.Threading.Thread.Sleep(150);
                  InputState = InputState.Up;
                }
                first = true;
              }
            }
          }
        }
      }

      CPUMode = CPUMode.Reset;
      MemoryProtect = MemoryProtect.Enabled;
    }
  }
}
