﻿using System;
using System.IO;
using System.IO.Ports;

namespace Core
{
  public class SerialControlPort : ControlPort
  {
    private SerialPort serialPort;

#if DEBUGLOG
    private System.IO.StreamWriter logger;
#endif

    public SerialControlPort()
    {
      CharacterDelay = 125;
      serialPort = new SerialPort(GetSerialPortFromFile());
      serialPort.DataReceived += new SerialDataReceivedEventHandler(serialPort_DataReceived);

#if DEBUGLOG
      logger = System.IO.File.CreateText("portLog.txt");
      logger.AutoFlush = true;
#endif
    }

    string GetSerialPortFromFile()
    {
      return File.ReadAllText("SerialPort.txt");
    }

    void serialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
    {
      if (e.EventType == SerialData.Chars)
      {
        string inputLine = serialPort.ReadLine();
        switch (inputLine[0])
        {
          case 'I':
            if (inputLine[1] == 'U')
            {
              InputState = InputState.Up;
            }
            else
            {
              InputState = InputState.Down;
            }
            break;
          case 'L':
            CPUMode = CPUMode.Load;
            break;
          case 'M':
            if (inputLine[1] == 'W')
            {
              MemoryProtect = MemoryProtect.Disabled;
            }
            else
            {
              MemoryProtect = MemoryProtect.Enabled;
            }
            break;
          case 'R':
            if (inputLine[1] == 'U')
            {
              CPUMode = CPUMode.Run;
            }
            else
            {
              CPUMode = CPUMode.Reset;
            }
            break;
          default:
            Data = inputLine;
            break;
        }
      }
    }

    public int CharacterDelay { get; set; }

    public string PortName
    {
      get { return serialPort.PortName; }
      set { serialPort.PortName = value; }
    }

    public int BaudRate
    {
      get { return serialPort.BaudRate; }
      set { serialPort.BaudRate = value; }
    }

    public override void SetCPUMode(CPUMode mode)
    {
      switch (mode)
      {
        case CPUMode.Load:
          WriteData("LD");
          break;
        case CPUMode.Reset:
          WriteData("RE");
          break;
        case CPUMode.Run:
          WriteData("GO");
          break;
      }
    }

    public override void SetINPState(InputState state)
    {
      if (state == InputState.Down)
      {
        WriteData("ID");
      }
      else
      {
        WriteData("IU");
      }
    }

    public override void SetMPState(MemoryProtect mp)
    {
      if (mp == MemoryProtect.Disabled)
      {
        WriteData("MW");
      }
      else
      {
        WriteData("MP");
      }
    }

    public override void Start()
    {
      if (!serialPort.IsOpen)
      {
        serialPort.Open();
      }
    }

    public override void Stop()
    {
      if (serialPort.IsOpen)
      {
        serialPort.Close();
      }
    }

    public override void WriteData(string data)
    {
      if (!serialPort.IsOpen)
      {
        serialPort.Open();
      }

#if DEBUGLOG
      logger.WriteLine(data);
      logger.Flush();
#endif

      foreach (char c in data)
      {
        serialPort.Write(c.ToString().ToUpperInvariant());
        System.Threading.Thread.Sleep(125);
      }
    }
  }
}
