﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Core;

namespace UI
{
  public partial class ControlKeys : UserControl
  {
    public ControlKeys()
    {
      InitializeComponent();
      SetMPState(MemoryProtect.Enabled);
      SetCPUMode(CPUMode.Reset);
    }

    public event EventHandler<CPUModeEventArgs> CPUModeClicked;
    public event EventHandler MPClicked;

    private void MP_clicked(object sender, EventArgs e)
    {
      OnMPClicked();
    }

    private void State_clicked(object sender, EventArgs e)
    {
      Button key = sender as Button;
      switch (key.Text)
      {
        case "R":
          OnCPUModeClicked(new CPUModeEventArgs(CPUMode.Reset));
          break;
        case "L":
          OnCPUModeClicked(new CPUModeEventArgs(CPUMode.Load));
          break;
        case "G":
          OnCPUModeClicked(new CPUModeEventArgs(CPUMode.Run));
          break;
      }
    }

    protected virtual void OnCPUModeClicked(CPUModeEventArgs e)
    {
      if (CPUModeClicked != null)
      {
        CPUModeClicked(this, e);
      }
    }

    protected virtual void OnMPClicked()
    {
      if (MPClicked != null)
      {
        MPClicked(this, EventArgs.Empty);
      }
    }

    public void SetCPUMode(CPUMode mode)
    {
      ledReset.Active = mode == CPUMode.Reset;
      ledGo.Active = mode == CPUMode.Run;
      ledLoad.Active = mode == CPUMode.Load;
    }

    public void SetMPState(MemoryProtect mpState)
    {
      ledMP.Active = mpState == MemoryProtect.Enabled;
      buttonMP.Text = ((mpState == MemoryProtect.Enabled) ? "MP" : "MW");
    }
  }
}
