﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

using Core;

namespace UI
{
  public partial class MainView : Form, IView
  {
    public MainView()
    {
      InitializeComponent();
      textInfo.Text = "RESET";
      
      keypad.ValueChanged += new EventHandler(keypad_ValueChanged);

      controlKeys.CPUModeClicked += new EventHandler<CPUModeEventArgs>(controlKeys_CPUModeClicked);
      controlKeys.MPClicked += new EventHandler(controlKeys_MPClicked);
    }

    void controlKeys_MPClicked(object sender, EventArgs e)
    {
      if (Presenter.MemoryProtect == MemoryProtect.Enabled)
      {
        Presenter.MemoryProtect = MemoryProtect.Disabled;
      }
      else
      {
        Presenter.MemoryProtect = MemoryProtect.Enabled;
      }
    }

    void controlKeys_CPUModeClicked(object sender, CPUModeEventArgs e)
    {
      Presenter.CPUMode = e.CPUMode;
    }

    void keypad_ValueChanged(object sender, EventArgs e)
    {
      Presenter.SendData(keypad.Value);
    }

    #region IView Members

    public Presenter Presenter { get; set; }

    public void SetCPUMode(object sender, EventArgs e)
    {
      if (InvokeRequired)
      {
        Invoke(new EventHandler(SetCPUMode), sender, e);
      }
      controlKeys.SetCPUMode(Presenter.CPUMode);
      switch (Presenter.CPUMode)
      {
        case CPUMode.Load:
          if (Presenter.MemoryProtect == MemoryProtect.Enabled)
          {
            textInfo.Text = "READING";
          }
          else
          {
            textInfo.Text = "LOADING";
          }
          break;
        case CPUMode.Reset:
          textInfo.Text = "RESET";
          break;
        case CPUMode.Run:
          textInfo.Text = "RUNNING";
          break;
      }
    }

    public void SetMPMode(object sender, EventArgs e)
    {
      if (InvokeRequired)
      {
        Invoke(new EventHandler(SetMPMode), sender, e);
      }
      controlKeys.SetMPState(Presenter.MemoryProtect);

      if (Presenter.CPUMode == CPUMode.Load)
      {
        textInfo.Text = Presenter.MemoryProtect == MemoryProtect.Enabled ? "READING" : "LOADING";
      }
    }

    public void SetInputState(object sender, EventArgs e)
    {
      if (InvokeRequired)
      {
        Invoke(new EventHandler(SetInputState), sender, e);
      }
      if (Presenter.InputState == InputState.Down)
      {
        buttonINP.BackColor = Color.LimeGreen;
      }
      else
      {
        buttonINP.BackColor = Color.ForestGreen;
      }
    }

    public void SetDisplay(object sender, EventArgs e)
    {
      if (InvokeRequired)
      {
        Invoke(new EventHandler(SetDisplay), sender, e);
      }

      hexDisplay.Text = Presenter.Data;
    }

    public void SetQ(object sender, EventArgs e)
    {
      if (InvokeRequired)
      {
        Invoke(new EventHandler(SetQ), sender, e);
      }
      ledQ.Active = Presenter.QState;
    }

    #endregion

    private void File_Click(object sender, EventArgs e)
    {
      DialogResult result = openFileDialog.ShowDialog();
      if (result == DialogResult.OK)
      {
        Presenter.LoadFromFile(openFileDialog.FileName);
      }
    }

    private void buttonINP_MouseDown(object sender, MouseEventArgs e)
    {
      Presenter.InputState = InputState.Down;
    }

    private void buttonINP_MouseUp(object sender, MouseEventArgs e)
    {
      Presenter.InputState = InputState.Up;
    }

    private void buttonOK_Click(object sender, EventArgs e)
    {
      panelOptions.Visible = false;
    }

    private void buttonOptions_Click(object sender, EventArgs e)
    {
      panelOptions.Visible = true;
    }
  }
}
